/*
 * This module implements the decoding of Rx packets
 * into human-readable form.
 */

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include "../include/pktmux.h"
#include "../include/limits.h"

extern u_char rxpkt[];
extern size_t rxpkt_len;

static char fmtbuf[MAX_PKT_FROM_TARGET*8];	/* size it generously */

void
print_rv_trace()
{
	int i, c;
	char *dp;

	dp = fmtbuf;
	strcpy(dp, "RV ");
	dp += 3;
	/* the SWE static ID is sent MSB first */
	for (i = 1; i <= 4; i++) {
		sprintf(dp, "%02X", rxpkt[i]);
		dp += 2;
	}
	/* severity level */
	sprintf(dp, " %d ", rxpkt[5]);
	dp = index(dp, '\0');
	for (i = 6; i < rxpkt_len; i++) {
		c = rxpkt[i];
		if (c & 0x80) {
			*dp++ = 'M';
			*dp++ = '-';
			c &= 0x7F;
		}
		if (c < 0x20) {
			*dp++ = '^';
			*dp++ = c + '@';
		} else if (c == 0x7F) {
			*dp++ = '^';
			*dp++ = '?';
		} else
			*dp++ = c;
	}
	*dp = '\0';
	output_line(fmtbuf);
}

void
print_l1_trace()
{
	int i, c;
	char *dp;

	dp = fmtbuf;
	strcpy(dp, "L1: ");
	dp += 4;
	for (i = 1; i < rxpkt_len; i++) {
		if ((i+1 < rxpkt_len) &&
		    (rxpkt[i] == '\r' && rxpkt[i+1] == '\n' ||
		     rxpkt[i] == '\n' && rxpkt[i+1] == '\r')) {
			*dp = '\0';
			output_line(fmtbuf);
			if (i+2 == rxpkt_len)
				return;
			dp = fmtbuf;
			*dp++ = '+';
			*dp++ = ' ';
			i++;
			continue;
		}
		c = rxpkt[i];
		if (c & 0x80) {
			*dp++ = 'M';
			*dp++ = '-';
			c &= 0x7F;
		}
		if (c < 0x20) {
			*dp++ = '^';
			*dp++ = c + '@';
		} else if (c == 0x7F) {
			*dp++ = '^';
			*dp++ = '?';
		} else
			*dp++ = c;
	}
	/* will get here only if no newline sequence at the end */
	*dp = '\0';
	output_line(fmtbuf);
}

void
print_g23_trace()
{
	/* messy logic factored out into libg23 */
	format_g23_packet(rxpkt, (int)rxpkt_len, fmtbuf);
	output_line(fmtbuf);
}

void
print_etm_output_raw()
{
	int i;
	char *dp;

	dp = fmtbuf;
	strcpy(dp, "ETM:");
	dp += 4;
	for (i = 1; i < rxpkt_len; i++) {
		sprintf(dp, " %02X", rxpkt[i]);
		dp += 3;
	}
	*dp = '\0';
	output_line(fmtbuf);
}

void
print_unknown_packet()
{
	int i;
	char *dp;

	dp = fmtbuf;
	strcpy(dp, "UNK:");
	dp += 4;
	for (i = 0; i < rxpkt_len; i++) {
		sprintf(dp, " %02X", rxpkt[i]);
		dp += 3;
	}
	*dp = '\0';
	output_line(fmtbuf);
}
